import logging

import pyotp
from discord.ext import commands
from googleapiclient.errors import HttpError

from les_louisdelatech.models.otp import Otp
from les_louisdelatech.utils.discord import is_team_allowed
from les_louisdelatech.utils.gsuite import format_google_api_error, search_user
from les_louisdelatech.utils.LouisDeLaTechError import LouisDeLaTechError
from les_louisdelatech.utils.User import User

logger = logging.getLogger()


class OtpCog(commands.Cog):
    def __init__(self, bot):
        self.bot = bot

    @commands.hybrid_command(name="lotp", help="List otp code")
    @commands.guild_only()
    @is_team_allowed
    async def list_otp(self, ctx):
        await ctx.defer()
        try:
            user = User(
                search_user(self.bot.admin_sdk(), ctx.author.name, ctx.author.id)
            )
        except LouisDeLaTechError as e:
            await ctx.send(f"{ctx.author} => {e.args[0]}")
            return
        except HttpError as e:
            await ctx.send(format_google_api_error(e))
            raise

        otps = await Otp.filter(team=user.team)

        message = f"Otp code available for team {user.team} :\n```"
        if len(otps) > 0:
            for o in otps:
                message += f"\n{o.name}"
        else:
            message += "No Otp code available"
        message += "```"

        await ctx.send(message)

    @commands.hybrid_command(name="gotp", help="Get otp code")
    @commands.guild_only()
    @is_team_allowed
    async def get_otp(
        self, ctx, name: str = commands.parameter(description="Otp name")
    ):
        await ctx.defer()
        try:
            user = User(
                search_user(self.bot.admin_sdk(), ctx.author.name, ctx.author.id)
            )
        except LouisDeLaTechError as e:
            await ctx.send(f"{ctx.author} => {e.args[0]}")
            return
        except HttpError as e:
            await ctx.send(format_google_api_error(e))
            raise

        otp = await Otp.get(name=name, team=user.team)
        totp = pyotp.TOTP(
            s=self.bot.decrypt(otp.secret),
            digest=otp.digest,
            digits=otp.digits,
            name=otp.name,
        )

        await ctx.author.send(f"Otp code for {name} is {totp.now()}")

        logger.info(f"Otp code {name} of team {user.team} send in DM to {ctx.author}")
        await ctx.send(
            f"Otp code {name} of team {user.team} send in DM to {ctx.author}"
        )

    @commands.hybrid_command(name="cotp", help="Create otp code")
    @commands.guild_only()
    @is_team_allowed
    async def create_otp(
        self,
        ctx,
        name: str = commands.parameter(description="Otp name"),
        digest: str = commands.parameter(description="Otp digest"),
        digits: str = commands.parameter(description="Otp digits"),
        secret: str = commands.parameter(description="Otp secret"),
    ):
        await ctx.defer()
        await ctx.message.delete()

        try:
            user = User(
                search_user(self.bot.admin_sdk(), ctx.author.name, ctx.author.id)
            )
        except LouisDeLaTechError as e:
            await ctx.send(f"{ctx.author} => {e.args[0]}")
            return
        except HttpError as e:
            await ctx.send(format_google_api_error(e))
            raise

        await Otp.create(
            name=name,
            team=user.team,
            digest=digest,
            digits=digits,
            secret=self.bot.encrypt(secret),
        )

        logger.info(f"Otp code {name} of team {user.team} created by {ctx.author}")
        await ctx.send(f"Otp code {name} of team {user.team} created by {ctx.author}")

    @commands.hybrid_command(name="dotp", help="Delete otp code")
    @commands.guild_only()
    @is_team_allowed
    async def delete_otp(
        self, ctx, name: str = commands.parameter(description="Otp name")
    ):
        await ctx.defer()
        try:
            user = User(
                search_user(self.bot.admin_sdk(), ctx.author.name, ctx.author.id)
            )
        except LouisDeLaTechError as e:
            await ctx.send(f"{ctx.author} => {e.args[0]}")
            return
        except HttpError as e:
            await ctx.send(format_google_api_error(e))
            raise

        await Otp.filter(name=name, team=user.team).delete()

        logger.info(f"Otp code {name} of team {user.team} deleted by {ctx.author}")
        await ctx.send(f"Otp code {name} of team {user.team} deleted by {ctx.author}")


async def setup(bot):
    await bot.add_cog(OtpCog(bot))



import logging
import os
import time

import discord
from discord.ext import commands
from discord.utils import escape_markdown, get
from googleapiclient.errors import HttpError
from jinja2 import Template

from les_louisdelatech.utils.gsuite import (
    add_user,
    add_user_group,
    delete_user_group,
    format_google_api_error,
    get_users,
    is_gsuite_admin,
    is_user_managed,
    search_user,
    suspend_user,
    update_user_department,
    update_user_password,
    update_user_pseudo,
    update_user_recovery,
    update_user_signature,
)
from les_louisdelatech.utils.LouisDeLaTechError import LouisDeLaTechError
from les_louisdelatech.utils.password import generate_password
from les_louisdelatech.utils.User import User

logger = logging.getLogger()


class UserCog(commands.Cog):
    @commands.hybrid_command(help="Provision an user")
    @commands.guild_only()
    @is_gsuite_admin
    async def provision(
        self,
        ctx,
        member: discord.Member = commands.parameter(description="Discord user"),
        firstname: str = commands.parameter(description="User firstname"),
        lastname: str = commands.parameter(description="User lastname"),
        pseudo: str = commands.parameter(description="User pseudo"),
        role_name: str = commands.parameter(description="User role"),
    ):
        """
        Provision an user
        [Discord]
            => User will be added to default group
            => User will be added to team group
        [Google]
            => User will be created and added to team group
        """
        await ctx.defer()
        user_email = User.email_from_name(firstname, lastname)
        user_team = self.bot.config["teams"].get(role_name, None)
        password = generate_password()
        admin_sdk = self.bot.admin_sdk()
        signature_template = Template(
            open(
                os.path.join(
                    self.bot.root_dir, "./templates/google/gmail_signature.j2"
                ),
                encoding="utf-8",
            ).read()
        )

        if user_team is None:
            await ctx.send(f"Role {role_name} is not managed by bot")
            return
        elif not user_team["team_role"]:
            await ctx.send(f"Role {role_name} is not a team role")
            return

        try:
            add_user(
                admin_sdk,
                firstname,
                lastname,
                user_email,
                password,
                role_name,
                member.id,
                pseudo,
            )
            add_user_group(admin_sdk, user_email, user_team["google_email"])

            # force time sleep or refresh token will cause an error
            # maybe API caching issue (if request is too fast)
            time.sleep(5)

            update_user_signature(
                self.bot.gmail_sdk(user_email),
                signature_template,
                user_email,
                firstname,
                lastname,
                None,
                role_name,
                user_team["team_role"],
            )
        except HttpError as e:
            await ctx.send(format_google_api_error(e))
            raise

        for role_name in self.bot.config["discord"]["roles"]["default"]:
            role = get(member.guild.roles, name=role_name)
            if role:
                await member.add_roles(role)
            else:
                await ctx.send(
                    f"Discord role {role_name} does not exist on server, check bot config"
                )
                return
        role = get(member.guild.roles, name=user_team["discord"])
        if role:
            await member.add_roles(role)
        else:
            await ctx.send(f"Discord role {role_name} does not exist on discord server")
            return

        await member.edit(nick=User.discord_name(firstname, pseudo, lastname))

        await ctx.send(f"User {user_email} provisionned")

        template = Template(
            open(
                os.path.join(self.bot.root_dir, "./templates/discord/base.j2"),
                encoding="utf-8",
            ).read()
        )
        await member.send(
            template.render(
                {"email": user_email, "password": escape_markdown(password)}
            )
        )

        template = Template(
            open(
                os.path.join(
                    self.bot.root_dir,
                    f"./templates/discord/{user_team['message_template']}",
                ),
                encoding="utf-8",
            ).read()
        )
        team_message = template.render()
        if team_message:
            await member.send(team_message)

    def __init__(self, bot):
        self.bot = bot

    @commands.hybrid_command(help="Deprovision an user")
    @commands.guild_only()
    @is_gsuite_admin
    async def deprovision(
        self,
        ctx,
        member: discord.Member = commands.parameter(description="Discord user"),
    ):
        """
        [Discord]
            => User will be removed from all groups
        [Google]
            => User will be suspended
        """
        await ctx.defer()
        try:
            user = User(search_user(self.bot.admin_sdk(), member.name, member.id))
            is_user_managed(
                self.bot.admin_sdk(), user.email, self.bot.config["teams_to_skip"]
            )
        except LouisDeLaTechError as e:
            await ctx.send(f"{member} => {e.args[0]}")
            return
        except HttpError as e:
            await ctx.send(format_google_api_error(e))
            raise

        try:
            suspend_user(self.bot.admin_sdk(), user.email)
        except HttpError as e:
            await ctx.send(format_google_api_error(e))
            raise

        await member.edit(roles=[])

        await ctx.send(f"User {member.name} deprovisionned")

    @commands.hybrid_command(name="uteam", help="Update user team")
    @commands.guild_only()
    @is_gsuite_admin
    async def update_team(
        self,
        ctx,
        member: discord.Member = commands.parameter(description="Discord user"),
        new_team_name: str = commands.parameter(description="New team name"),
    ):
        """
        [Discord]
            => User will be removed from all team groups
            => User will be added to this new team
        [Google]
            => User will be removed from all team groups
            => User will be added to this new team
            => User signature will be updated
        """
        await ctx.defer()
        try:
            user = User(search_user(self.bot.admin_sdk(), member.name, member.id))
            is_user_managed(
                self.bot.admin_sdk(), user.email, self.bot.config["teams_to_skip"]
            )
            user.team = new_team_name
        except LouisDeLaTechError as e:
            await ctx.send(f"{member} => {e.args[0]}")
            return
        except HttpError as e:
            await ctx.send(format_google_api_error(e))
            raise

        new_user_team = self.bot.config["teams"].get(user.team, None)
        admin_sdk = self.bot.admin_sdk()
        signature_template = Template(
            open(
                os.path.join(
                    self.bot.root_dir, "./templates/google/gmail_signature.j2"
                ),
                encoding="utf-8",
            ).read()
        )

        if new_user_team is None:
            await ctx.send(f"Role {new_team_name} does not exist, check bot config")
            return
        elif not new_user_team["team_role"]:
            await ctx.send(f"Role {new_user_team} is invalid, check bot config")
            return

        try:
            for v in self.bot.config["teams"].values():
                delete_user_group(admin_sdk, user.email, v["google_email"])
            add_user_group(admin_sdk, user.email, new_user_team["google_email"])
            update_user_department(admin_sdk, user.email, user.team)
            update_user_signature(
                self.bot.gmail_sdk(user.email),
                signature_template,
                user.email,
                user.firstname,
                user.lastname,
                user.role,
                user.team,
                new_user_team["team_role"],
            )
        except HttpError as e:
            await ctx.send(format_google_api_error(e))
            raise

        if new_user_team is None:
            await ctx.send(f"Role {user.team} does not exist, check bot config")
            return

        for v in self.bot.config["teams"].values():
            role = get(member.guild.roles, name=v["discord"])
            if role:
                await member.remove_roles(role)
            else:
                await ctx.send(
                    f"Discord role {v['discord']} does not exist, check bot config"
                )
                return
        role = get(member.guild.roles, name=new_user_team["discord"])
        if role:
            await member.add_roles(role)
        else:
            await ctx.send(f"Discord role {user.team} does not exist")
            return

        await ctx.send(f"User {member.name} is now member of team: {user.team}")

    @commands.hybrid_command(name="upseudo", help="Update user pseudo")
    @commands.guild_only()
    @is_gsuite_admin
    async def update_pseudo(
        self,
        ctx,
        member: discord.Member = commands.parameter(description="Discord user"),
        new_pseudo: str = commands.parameter(description="New user pseudo"),
    ):
        """
        [Discord]
            => User will be renamed
        [Google]
            => User pseudo will be renamed
        """
        await ctx.defer()
        try:
            user = User(search_user(self.bot.admin_sdk(), member.name, member.id))
            is_user_managed(
                self.bot.admin_sdk(), user.email, self.bot.config["teams_to_skip"]
            )
            user.pseudo = new_pseudo
        except LouisDeLaTechError as e:
            await ctx.send(f"{member} => {e.args[0]}")
            return
        except HttpError as e:
            await ctx.send(format_google_api_error(e))
            raise

        try:
            update_user_pseudo(
                self.bot.admin_sdk(),
                user.email,
                user.pseudo,
            )
        except HttpError as e:
            await ctx.send(format_google_api_error(e))
            raise

        old_nick = member.nick

        await member.edit(
            nick=User.discord_name(user.firstname, user.pseudo, user.lastname)
        )

        await ctx.send(f"User {old_nick} you now shall be called {member.nick} !")

    @commands.hybrid_command(
        name="usignatures", help="Update the signature of all users on gmail"
    )
    @commands.guild_only()
    @is_gsuite_admin
    async def update_signatures(self, ctx):
        await ctx.defer()
        user_updated = 0
        try:
            users = get_users(self.bot.admin_sdk())
        except HttpError as e:
            await ctx.send(format_google_api_error(e))
            raise

        signature_template = Template(
            open(
                os.path.join(
                    self.bot.root_dir, "./templates/google/gmail_signature.j2"
                ),
                encoding="utf-8",
            ).read()
        )

        await ctx.send(f"Starting to update {len(users)} users")
        for _user in users:
            try:
                user = User(_user)
                is_user_managed(
                    self.bot.admin_sdk(), user.email, self.bot.config["teams_to_skip"]
                )
                user_team = self.bot.config["teams"].get(user.team, None)
                update_user_signature(
                    self.bot.gmail_sdk(user.email),
                    signature_template,
                    user.email,
                    user.firstname,
                    user.lastname,
                    user.role,
                    user.team,
                    user_team["team_role"],
                )
                user_updated += 1
            except LouisDeLaTechError as e:
                await ctx.send(f"{e.args[0]}")
                continue
            except HttpError as e:
                await ctx.send(format_google_api_error(e))
                return

        await ctx.send(f"Updated signatures for {user_updated}/{len(users)} users")

    @commands.hybrid_command(help="Update recovery email of an user")
    @commands.guild_only()
    @is_gsuite_admin
    async def urecovery(
        self,
        ctx,
        member: discord.Member = commands.parameter(description="Discord user"),
        backup_email: str = commands.parameter(description="User backup email"),
    ):
        await ctx.defer()
        try:
            user = User(search_user(self.bot.admin_sdk(), member.name, member.id))
            is_user_managed(
                self.bot.admin_sdk(), user.email, self.bot.config["teams_to_skip"]
            )
        except LouisDeLaTechError as e:
            await ctx.send(f"{member} => {e.args[0]}")
            return
        except HttpError as e:
            await ctx.send(format_google_api_error(e))
            raise
        try:
            update_user_recovery(self.bot.admin_sdk(), user.email, backup_email)
        except HttpError as e:
            await ctx.send(format_google_api_error(e))
            raise

        await ctx.send(f"Updated recovery information for user {member.name}")

    @commands.hybrid_command(help="Reset password of an user")
    @commands.guild_only()
    @is_gsuite_admin
    async def rpassword(
        self,
        ctx,
        member: discord.Member = commands.parameter(description="Discord user"),
    ):
        await ctx.defer()
        try:
            user = User(search_user(self.bot.admin_sdk(), member.name, member.id))
            is_user_managed(
                self.bot.admin_sdk(), user.email, self.bot.config["teams_to_skip"]
            )
        except LouisDeLaTechError as e:
            await ctx.send(f"{member} => {e.args[0]}")
            return
        except HttpError as e:
            await ctx.send(format_google_api_error(e))
            raise

        temp_pass = generate_password()

        template = Template(
            open(
                os.path.join(
                    self.bot.root_dir, "./templates/discord/reset_password.j2"
                ),
                encoding="utf-8",
            ).read()
        )

        try:
            update_user_password(self.bot.admin_sdk(), user.email, temp_pass, True)
        except HttpError as e:
            await ctx.send(format_google_api_error(e))
            raise

        await member.send(
            template.render(
                {"email": user.email, "password": escape_markdown(temp_pass)}
            )
        )
        await ctx.send(f"Sent a new password to {member.name} in PM")


async def setup(bot):
    await bot.add_cog(UserCog(bot))



import logging

import discord
import httpx
from discord.ext import commands
from discord.ext.commands import Context

logger = logging.getLogger()


class CatCog(commands.Cog):
    def __init__(self, bot):
        self.bot = bot

    @commands.hybrid_command(name="cat", help="Get cat")
    async def get_cat(self, ctx: Context):
        await ctx.defer()
        async with httpx.AsyncClient() as client:
            cat_data = await client.get("https://api.thecatapi.com/v1/images/search")

        payload = cat_data.json()
        if not payload:
            return
        cat_url = payload[0]["url"]
        embed = discord.Embed(
            colour=discord.Colour.random(),
            title="Here is your kitty",
        ).set_image(url=cat_url)
        await ctx.send(embed=embed)


async def setup(bot):
    await bot.add_cog(CatCog(bot))



import logging
from functools import wraps
from http.client import responses

from les_louisdelatech.utils.LouisDeLaTechError import LouisDeLaTechError
from les_louisdelatech.utils.password import hash_password
from les_louisdelatech.utils.User import User

logger = logging.getLogger(__name__)


def is_gsuite_admin(func):
    @wraps(func)
    async def wrapper(self, ctx, *args, **kwargs):
        try:
            user = User(
                search_user(self.bot.admin_sdk(), ctx.author.name, ctx.author.id)
            )
        except LouisDeLaTechError as e:
            await ctx.send(f"{ctx.author} {e.args[0]}")
            return None
        if not user.is_admin:
            await ctx.send(f"{ctx.author} you are not a Gsuite admin")
            logger.error(f"{ctx.author} you are not a Gsuite admin")
            return None
        return await func(self, ctx, *args, **kwargs)

    return wrapper


def format_google_api_error(error):
    return f"Google API error status code {error.status_code}:{responses[error.status_code]}"


def is_user_managed(admin_sdk, user_email, teams_to_skip):
    user = make_request(
        admin_sdk.users().get(
            userKey=user_email, projection="full", viewType="admin_view"
        )
    )

    if user is None:
        raise LouisDeLaTechError(f"No Gsuite account found for user: {user_email}")

    if User(user).team in teams_to_skip:
        raise LouisDeLaTechError(
            f"Gsuite account not managed by this bot for this user: {user_email}"
        )


def user_is_in_group(admin_sdk, user_email, group_email):
    return make_request(
        admin_sdk.members().hasMember(groupKey=group_email, memberKey=user_email)
    )["isMember"]


def get_users(admin_sdk):
    users = []
    resp = {"nextPageToken": None}
    while "nextPageToken" in resp:
        resp = make_request(
            admin_sdk.users().list(
                domain="lyon-esport.fr",
                projection="full",
                viewType="admin_view",
                pageToken=resp["nextPageToken"]
                if "nextPageToken" in resp and resp["nextPageToken"] is not None
                else None,
            )
        )
        users += resp["users"]

    return users


def search_user(admin_sdk, discord_pseudo, discord_id):
    users = make_request(
        admin_sdk.users().list(
            query=f"custom.discordId={discord_id}",
            customer="my_customer",
            projection="full",
            viewType="admin_view",
        )
    )

    users = users["users"] if "users" in users else []

    if len(users) == 0:
        raise LouisDeLaTechError(
            f"No Gsuite account found with discordId: {discord_id} for user {discord_pseudo}"
        )
    elif len(users) > 1:
        raise LouisDeLaTechError(
            f"Multiple Gsuite users with same discordId: {discord_id} for user {discord_pseudo}"
        )

    return users[0]


def add_user(
    admin_sdk, firstname, lastname, email, password, group, discord_id, pseudo
):
    body = {
        "name": {
            "familyName": lastname,
            "givenName": firstname,
            "fullName": f"{firstname.title()} {lastname.title()}",
        },
        "primaryEmail": email,
        "customSchemas": {
            "custom": {
                "discordId": discord_id,
                "pseudo": pseudo,
            },
        },
        "organizations": [{"primary": True, "customType": "", "department": group}],
        "password": hash_password(password),
        "hashFunction": "crypt",
        "changePasswordAtNextLogin": True,
    }
    make_request(admin_sdk.users().insert(body=body))


def update_user_pseudo(admin_sdk, user_email, pseudo):
    body = {
        "customSchemas": {
            "custom": {
                "pseudo": pseudo,
            },
        },
    }
    make_request(admin_sdk.users().update(userKey=user_email, body=body))


def update_user_signature(
    gmail_sdk, template, user_email, firstname, lastname, role, team, team_role
):
    make_request(
        gmail_sdk.users()
        .settings()
        .sendAs()
        .update(
            userId=user_email,
            sendAsEmail=user_email,
            body={
                "signature": template.render(
                    {
                        "email": user_email,
                        "firstname": firstname,
                        "lastname": lastname,
                        "role": role,
                        "team": team if team_role else None,
                    }
                )
            },
        )
    )


def suspend_user(admin_sdk, user_email):
    body = {"suspended": True}
    make_request(admin_sdk.users().update(userKey=user_email, body=body))


def update_user_department(admin_sdk, user_email, department):
    body = {
        "organizations": [{"primary": True, "customType": "", "department": department}]
    }
    make_request(admin_sdk.users().update(userKey=user_email, body=body))


def update_user_password(admin_sdk, user_email, password, temporary_pass):
    body = {
        "password": hash_password(password),
        "hashFunction": "crypt",
        "changePasswordAtNextLogin": temporary_pass,
    }
    make_request(admin_sdk.users().update(userKey=user_email, body=body))


def update_user_recovery(admin_sdk, user_email, recovery_email):
    body = {"recoveryEmail": recovery_email}
    make_request(admin_sdk.users().update(userKey=user_email, body=body))


def add_user_group(admin_sdk, user_email, group_email):
    body = {
        "email": user_email,
        "role": "MEMBER",
    }
    make_request(admin_sdk.members().insert(groupKey=group_email, body=body))


def delete_user_group(admin_sdk, user_email, group_email):
    if user_is_in_group(admin_sdk, user_email, group_email):
        make_request(
            admin_sdk.members().delete(groupKey=group_email, memberKey=user_email)
        )


def make_request(req):
    return req.execute()



import logging
from argparse import ArgumentParser

import sentry_sdk
import tomli

from les_louisdelatech.bot import LouisDeLaTech

logging.basicConfig(level=logging.INFO, format="%(asctime)s %(levelname)s:%(message)s")
logger = logging.getLogger()

parser = ArgumentParser()
parser.add_argument(
    "-c",
    "--config",
    action="store",
    dest="config",
    default="/etc/LouisDeLaTech/config.toml",
    help="Path to config file",
)
parser.add_argument(
    "-g",
    "--google",
    action="store",
    dest="google",
    default="/etc/LouisDeLaTech/google.json",
    help="Path to google secrets json",
)
args = parser.parse_args()

logger.info("Bot started")

with open(args.config, "rb") as f:
    config = tomli.load(f)
logger.info("Config loaded")

log_level = logging.getLevelName(config["log_level"])
logger.setLevel(log_level)
logger.info(f"Started bot with log level {logging.getLevelName(logger.level)}")

if len(config["sentry_dsn"]) > 0:
    sentry_sdk.init(
        config["sentry_dsn"],
        # Set traces_sample_rate to 1.0 to capture 100%
        # of transactions for performance monitoring.
        # We recommend adjusting this value in production.
        traces_sample_rate=0.5,
    )

bot = LouisDeLaTech(config, args.google)

bot.run(config["discord"]["token"], reconnect=True)



from discord.ext import commands

from les_louisdelatech.utils.discord import is_team_allowed


class ManagementCog(commands.Cog):
    def __init__(self, bot):
        self.bot = bot

    @commands.hybrid_command(help="Change channel topic")
    @is_team_allowed
    async def topic(
        self,
        ctx,
        description: str = commands.parameter(description="Topic description"),
    ):
        await ctx.defer()
        await ctx.channel.edit(topic=description)
        await ctx.send("Channel topic updated")

    # Meeting voice channel creation & deletion listener
    @commands.Cog.listener()
    async def on_voice_state_update(self, member, before, after):
        # Create voice channel block
        if (
            after.channel
            and after.channel.name
            == self.bot.config["discord"]["voice_channel_creation"][
                "trigger_channel_name"
            ]
            and not member.bot
        ):
            # List meeting channels already existing in the user's category and order it
            list_channels_name = []

            def predicate(channel):
                return channel.name.startswith(
                    self.bot.config["discord"]["voice_channel_creation"][
                        "new_channel_name"
                    ]
                )

            for channel in filter(predicate, after.channel.category.voice_channels):
                list_channels_name.append(channel.name)

            # Iterate through the existing channels (if they exist) to create an non-existing one
            new_channel_name = None
            channel_number = 1

            while new_channel_name is None:
                channel_name_check = f'{self.bot.config["discord"]["voice_channel_creation"]["new_channel_name"]} #{channel_number}'
                if (
                    channel_name_check not in list_channels_name
                    or not list_channels_name
                ):
                    new_channel_name = channel_name_check
                channel_number = channel_number + 1

            # Create the channel and move member
            new_channel = await member.guild.create_voice_channel(
                new_channel_name,
                category=after.channel.category,
                bitrate=self.bot.config["discord"]["voice_channel_creation"]["bitrate"],
            )
            await member.move_to(new_channel)

        # Delete voice channel block
        if (
            before.channel
            and before.channel.name.startswith(
                self.bot.config["discord"]["voice_channel_creation"]["new_channel_name"]
            )
            and not member.bot
            and not before.channel.members
        ):
            await before.channel.delete(reason="Channel is empty")


async def setup(bot):
    await bot.add_cog(ManagementCog(bot))



from discord.ext import commands


class ConfigCog(commands.Cog):
    def __init__(self, bot):
        self.bot = bot

    @commands.hybrid_command(name="gteams", help="Get available teams")
    async def get_teams(self, ctx):
        await ctx.defer()
        message = "Available roles :\n```"

        for team in self.bot.config["teams"]:
            message += f"\n{team}"

        message += "```"
        await ctx.send(message)


async def setup(bot):
    await bot.add_cog(ConfigCog(bot))



import crypt
import secrets
import string


def generate_password():
    alphabet = string.ascii_letters + string.digits + string.punctuation
    password = "".join(
        secrets.choice(alphabet) for _ in range(secrets.SystemRandom().randint(20, 30))
    )
    return password


def hash_password(password):
    return crypt.crypt(password, crypt.METHOD_SHA512)



import random

import discord
from discord.ext import commands, tasks


class TaskCog(commands.Cog):
    def __init__(self, bot):
        self.bot = bot
        self.change_bot_activity.start()

    @tasks.loop(minutes=10.0)
    async def change_bot_activity(self):
        activity = random.choice(self.bot.config["discord"]["bot_activity"])
        await self.bot.change_presence(
            activity=discord.Game(activity), status=discord.Status.online
        )

    @change_bot_activity.before_loop
    async def before_send(self):
        await self.bot.wait_until_ready()

        return


async def setup(bot):
    await bot.add_cog(TaskCog(bot))



from enum import Enum

from tortoise import fields
from tortoise.models import Model


class Digest(str, Enum):
    sha1 = "sha1"
    sha256 = "sha256"
    sha512 = "sha512"


class Otp(Model):
    name = fields.TextField(pk=True)
    team = fields.TextField()
    digest: Digest = fields.CharEnumField(Digest, default=Digest.sha1)
    digits = fields.SmallIntField(default=6)
    secret = fields.BinaryField()

    def __str__(self):
        return self.name



import logging
import os
import sys
import traceback

import discord
from cryptography.fernet import Fernet
from discord.ext import commands
from google.oauth2.service_account import Credentials
from googleapiclient import discovery
from tortoise import Tortoise, connections

logger = logging.getLogger()


class LouisDeLaTech(commands.Bot):
    def __init__(self, config, google_path):
        super().__init__(
            command_prefix=config["discord"]["command_prefix"],
            description="LouisDeLaTech is a discord bot manager for Lyon e-Sport",
            intents=discord.Intents(
                messages=True,
                message_content=True,
                guilds=True,
                voice_states=True,
                members=True,
            ),
        )
        # added to make sure that the command tree will be synced only once
        self.synced = False

        self.root_dir = os.path.dirname(os.path.abspath(__file__))

        self.config = config
        self.google_path = google_path

        self.fernet = Fernet(self.config["db"]["secret_key"])

    def encrypt(self, s):
        return self.fernet.encrypt(s.encode("ascii"))

    def decrypt(self, s):
        return self.fernet.decrypt(s).decode("ascii")

    async def setup_hook(self):
        for extension in self.config["discord"]["initial_cogs"]:
            await self.load_extension(extension)

        await Tortoise.init(
            db_url=f"sqlite://{self.config['db']['filename']}",
            modules={"models": ["les_louisdelatech.models"]},
        )
        await Tortoise.generate_schemas()

    def admin_sdk(self):
        creds = Credentials.from_service_account_file(
            self.google_path,
            scopes=self.config["google"]["scopes"]["admin"],
            subject=self.config["google"]["subject"],
        )

        return discovery.build(
            "admin", "directory_v1", credentials=creds, cache_discovery=False
        )

    def gmail_sdk(self, user):
        creds = Credentials.from_service_account_file(
            self.google_path,
            scopes=self.config["google"]["scopes"]["gmail"],
            subject=user,
        )

        return discovery.build("gmail", "v1", credentials=creds, cache_discovery=False)

    async def on_ready(self):
        logger.info(f"Logged in as: {self.user.name} - {self.user.id}")
        logger.info("Successfully logged in and booted...!")
        if not self.synced:  # check if slash commands have been synced
            await self.tree.sync()
            logger.info("Slash commands synced")

    async def on_command_error(self, ctx, error):
        if isinstance(error, discord.ext.commands.errors.CommandNotFound):
            await ctx.send("Command not found")
        elif isinstance(ctx, discord.Interaction):
            await ctx.response.send_message(
                f"Error while executing command => {error.__cause__}"
            )
        else:
            await ctx.send(f"Error while executing command => {error.__cause__}")
        traceback.print_exception(
            type(error), error, error.__traceback__, file=sys.stderr
        )

    async def close(self):
        await connections.close_all()
        super()



from les_louisdelatech.utils.LouisDeLaTechError import LouisDeLaTechError


class User:
    def __init__(self, user):
        """
        :param user: User object from google API
        """
        self.check_user_setup(user)
        self.firstname = user["name"]["givenName"]
        self.lastname = user["name"]["familyName"]
        self.pseudo = (
            user["customSchemas"]["custom"]["pseudo"]
            if "pseudo" in user["customSchemas"]["custom"]
            else None
        )
        self.discord_id = int(user["customSchemas"]["custom"]["discordId"])
        self.email = user["primaryEmail"]
        self.team = user["organizations"][0]["department"]
        self.role = self.get_role(user)
        self.is_admin = user["isAdmin"]
        self.is_suspended = user["suspended"]

    @property
    def firstname(self):
        return self._firstname

    @firstname.setter
    def firstname(self, value):
        self._firstname = value.lower()

    @property
    def lastname(self):
        return self._lastname

    @lastname.setter
    def lastname(self, value):
        self._lastname = value.lower()

    @property
    def team(self):
        return self._team

    @team.setter
    def team(self, value):
        if value:
            self._team = value.lower()
        else:
            self._team = None

    @property
    def role(self):
        return self._role

    @role.setter
    def role(self, value):
        if value:
            self._role = value.lower()
        else:
            self._role = None

    @classmethod
    def discord_name(cls, firstname, pseudo, lastname):
        return f"{firstname.title()} {pseudo} {lastname[:1].upper()}"

    @classmethod
    def get_role(cls, user):
        if "organizations" in user and "title" in user["organizations"][0]:
            title = user["organizations"][0]["title"]
        else:
            title = None
        return title

    @classmethod
    def email_from_name(cls, firstname, lastname):
        return f"{firstname.lower()}.{lastname.lower()}@lyon-esport.fr"

    @classmethod
    def check_user_setup(cls, user):
        print(user)
        if not user:
            raise LouisDeLaTechError("User not found, user is not setup on Gsuite")
        elif (
            "customSchemas" not in user
            or "custom" not in user["customSchemas"]
            or "discordId" not in user["customSchemas"]["custom"]
        ):
            raise LouisDeLaTechError(
                f"Discord ID not found, discordId is not setup on Gsuite for user: {user['primaryEmail']}"
            )
        elif (
            "organizations" not in user or "department" not in user["organizations"][0]
        ):
            raise LouisDeLaTechError(
                f"Department not found, department is not setup on Gsuite for user: {user['primaryEmail']}"
            )



import logging
from functools import wraps

logger = logging.getLogger(__name__)


def is_team_allowed(func):
    @wraps(func)
    async def wrapper(self, ctx, *args, **kwargs):
        for role in ctx.author.roles:
            if any(
                role.name == team["discord"]
                for _, team in self.bot.config["teams_to_skip"].items()
            ):
                await ctx.send(
                    f"{ctx.author} your team is not allowed to run this command"
                )
                logger.info(
                    f"{ctx.author} your team is not allowed to run this command"
                )
                return None
        return await func(self, ctx, *args, **kwargs)

    return wrapper
